<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wp_Webhotelier
 * @subpackage Wp_Webhotelier/admin
 * @author     WPlugged <support@wplugged.com>
 */
class Wp_Webhotelier_Admin
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version     = $version;

    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {
        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/wp-webhotelier-admin.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/wp-webhotelier-admin.js', array('jquery'), $this->version, false);
        wp_localize_script($this->plugin_name, 'dismissible_notice', array('nonce' => wp_create_nonce('dismissible-notice')));
    }

    /**
     * This function creates the plugin's options with the use
     * of Titan Framework.
     *
     * @return void
     */
    public function create_options()
    {

        // Initialize Titan with my special unique namespace
        $titan = TitanFramework::getInstance('wp-webhotelier');

        // Create my admin panel
        $adminPanel = $titan->createAdminPanel([
            'name' => 'WebHotelier' . ' ' . __('Options', 'wp-webhotelier')
        ]);

        $formTab = $adminPanel->createTab([
            'name' => __('Form Fields Settings', 'wp-webhotelier')
        ]);

        $customizationTab = $adminPanel->createTab([
            'name' => __('Form Customization Settings', 'wp-webhotelier')
        ]);

        $formTab->createOption([
            'name'        => __('WebHotelier URL', 'wp-webhotelier'),
            'id'          => 'wp-webhotelier-url',
            'type'        => 'text',
            'desc'        => __('This is the URL of the hotel\'s reservation page', 'wp-webhotelier'),
            'placeholder' => 'https://example.reserve-online-net'
        ]);

        $formTab->createOption([
            'name'    => __('Max Nights', 'wp-webhotelier'),
            'id'      => 'wp-webhotelier-max-nights',
            'type'    => 'number',
            'desc'    => __('The maximum number of nights one can choose to make a reservation', 'wp-webhotelier'),
            'default' => '7',
            'max'     => '30',
            'unit'    => __('Nights', 'wp-webhotelier')
        ]);

        $formTab->createOption([
            'name'    => __('Max Rooms', 'wp-webhotelier'),
            'id'      => 'wp-webhotelier-max-rooms',
            'type'    => 'number',
            'desc'    => __('The maximum number of rooms one can choose to make a reservation', 'wp-webhotelier'),
            'default' => '5',
            'max'     => '10',
            'unit'    => __('Rooms', 'wp-webhotelier')
        ]);

        $formTab->createOption([
            'name'    => __('Max Adults', 'wp-webhotelier'),
            'id'      => 'wp-webhotelier-max-adults',
            'type'    => 'number',
            'desc'    => __('The maximum number of adults one can choose to make a reservation', 'wp-webhotelier'),
            'default' => '10',
            'max'     => '50',
            'unit'    => __('Adults', 'wp-webhotelier')
        ]);

        $formTab->createOption([
            'name'    => __('Max Children', 'wp-webhotelier'),
            'id'      => 'wp-webhotelier-max-children',
            'type'    => 'number',
            'desc'    => __('The maximum number of children one can choose to make a reservation', 'wp-webhotelier'),
            'default' => '10',
            'max'     => '50',
            'unit'    => __('Children', 'wp-webhotelier')
        ]);

        $formTab->createOption([
            'name'    => __('Max Infants', 'wp-webhotelier'),
            'id'      => 'wp-webhotelier-max-infants',
            'type'    => 'number',
            'desc'    => __('The maximum number of infants one can choose to make a reservation', 'wp-webhotelier'),
            'default' => '10',
            'max'     => '50',
            'unit'    => __('Infants', 'wp-webhotelier')
        ]);

        $formTab->createOption([
            'type'      => 'save',
            'reset'     => __('Reset to default values', 'wp-webhotelier'),
            'use_reset' => true
        ]);

        $customizationTab->createOption([
            'name'    => __('Orientation', 'wp-webhotelier'),
            'id'      => 'wp-webhotelier-orientation',
            'type'    => 'radio-image',
            'options' => [
                'vertical'   => plugin_dir_url(__FILE__) . 'images/vertical-form.png',
                'horizontal' => plugin_dir_url(__FILE__) . 'images/horizontal-form.png',
                'fluid'      => plugin_dir_url(__FILE__) . 'images/fluid-form.png'
            ],
            'desc'    => __('Select the layout orientation', 'wp-webhotelier'),
            'default' => 'horizontal'
        ]);

        $customizationTab->createOption([
            'name'    => __('Replace Nights with Checkout Date', 'wp-webhotelier'),
            'id'      => 'wp-webhotelier-checkout-date',
            'type'    => 'enable',
            'default' => false
        ]);

        $customizationTab->createOption([
            'name'        => __('Custom CSS Class', 'wp-webhotelier'),
            'id'          => 'wp-webhotelier-css-class',
            'type'        => 'text',
            'desc'        => __('Add your custom CSS Class which will be assigned to the form element', 'wp-webhotelier'),
            'placeholder' => 'my-custom-css-class'
        ]);

        $customizationTab->createOption([
            'name' => __('Custom CSS', 'wp-webhotelier'),
            'id'   => 'wp-webhotelier-custom-css',
            'type' => 'code',
            'desc' => __('Put your Custom CSS rules here. Useful when you have added a Custom CSS Class previously', 'wp-webhotelier'),
            'lang' => 'css'
        ]);

        $customizationTab->createOption([
            'type'      => 'save',
            'reset'     => __('Reset to default values', 'wp-webhotelier'),
            'use_reset' => true
        ]);
    }

    /**
     * Adds the donation markup into the action
     *
     * @return void
     */
    public function add_donation_markup()
    {
        include __DIR__ . '/partials/wp-webhotelier-paypal-code.php';
    }

    /**
     * Adds a notice in the admin page informing the user of the purpose of the plugin
     *
     * @return void
     */
    public function add_notice_markup()
    {
        if (is_admin() &&
            isset($_GET['page']) &&
            strpos(sanitize_key($_GET['page']), 'webhotelier') !== false &&
            $this->is_admin_notice_active('disable-done-notice-forever')) {
            include __DIR__ . '/partials/wp-webhotelier-notice-markup.php';
        }
    }

    /**
     * Handles Ajax request to persist notices dismissal.
     * Uses check_ajax_referer to verify nonce.
     */
    public function dismiss_admin_notice()
    {
        $option_name        = sanitize_text_field($_POST['option_name']);
        $dismissible_length = sanitize_text_field($_POST['dismissible_length']);
        if ('forever' != $dismissible_length) {
            $dismissible_length = (0 == absint($dismissible_length)) ? 1 : $dismissible_length;
            $dismissible_length = strtotime(absint($dismissible_length) . ' days');
        }
        check_ajax_referer('dismissible-notice', 'nonce');
        $this->set_admin_notice_cache($option_name, $dismissible_length);
        wp_die();
    }

    /**
     * Is admin notice active?
     *
     * @param string $arg data-dismissible content of notice.
     *
     * @return bool
     */
    public function is_admin_notice_active($arg)
    {
        $array = explode('-', $arg);
        array_pop($array);
        $option_name = implode('-', $array);
        $db_record   = $this->get_admin_notice_cache($option_name);
        if ('forever' == $db_record) {
            return false;
        } elseif (absint($db_record) >= time()) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Returns admin notice cached timeout.
     *
     * @param string|bool $id admin notice name or false.
     *
     * @return array|bool The timeout. False if expired.
     */
    public function get_admin_notice_cache($id = false)
    {
        if (!$id) {
            return false;
        }
        $cache_key = $this->plugin_name . '-' . md5($id);
        $timeout   = get_site_option($cache_key);
        $timeout   = ('forever' === $timeout) ? time() + 60 : $timeout;
        if (empty($timeout) || time() > $timeout) {
            return false;
        }
        return $timeout;
    }

    /**
     * Sets admin notice timeout in site option.
     *
     * @param string      $id       Data Identifier.
     * @param string|bool $timeout  Timeout for admin notice.
     *
     * @return bool
     */
    public function set_admin_notice_cache($id, $timeout)
    {
        $cache_key = $this->plugin_name . '-' . md5($id);
        update_site_option($cache_key, $timeout);
        return true;
    }
}
