(function(wp) {
    /**
     * Registers a new block provided a unique name and an object defining its behavior.
     * @see https://github.com/WordPress/gutenberg/tree/master/blocks#api
     */
    var registerBlockType = wp.blocks.registerBlockType;
    /**
     * Returns a new element of given type. Element is an abstraction layer atop React.
     * @see https://github.com/WordPress/gutenberg/tree/master/element#element
     */
    var el = wp.element.createElement,
        registerBlockType = wp.blocks.registerBlockType,
        ServerSideRender = wp.components.ServerSideRender,
        TextControl = wp.components.TextControl,
        SelectControl = wp.components.SelectControl,
        RangeControl = wp.components.RangeControl,
        InspectorControls = wp.editor.InspectorControls;
    /**
     * Retrieves the translation of text.
     * @see https://github.com/WordPress/gutenberg/tree/master/i18n#api
     */
    var __ = wp.i18n.__;

    /**
     * Every block starts by registering a new block type definition.
     * @see https://wordpress.org/gutenberg/handbook/block-api/
     */
    registerBlockType("wp-webhotelier/block", {
        /**
         * This is the display title for your block, which can be translated with `i18n` functions.
         * The block inserter will show this name.
         */
        title: __("WebHotelier Form", "wp-webhotelier"),

        /**
         * Blocks are grouped into categories to help users browse and discover them.
         * The categories provided by core are `common`, `embed`, `formatting`, `layout` and `widgets`.
         */
        category: "widgets",

        /**
         * Optional block extended support features.
         */
        supports: {
            // Removes support for an HTML mode.
            html: false,
            customClassName: false
        },

        /**
         * The edit function describes the structure of your block in the context of the editor.
         * This represents what the editor will render when the block is used.
         * @see https://wordpress.org/gutenberg/handbook/block-edit-save/#edit
         *
         * @param {Object} [props] Properties passed from the editor.
         * @return {Element}       Element to render.
         */
        edit: function(props) {
            return [
                /*
                 * The ServerSideRender element uses the REST API to automatically call
                 * php_block_render() in your PHP code whenever it needs to get an updated
                 * view of the block.
                 */
                el(ServerSideRender, {
                    block: "wp-webhotelier/block",
                    attributes: props.attributes
                }),
                /*
                 * InspectorControls lets you add controls to the Block sidebar. In this case,
                 * we're adding a TextControl, which lets us edit the 'foo' attribute (which
                 * we defined in the PHP). The onChange property is a little bit of magic to tell
                 * the block editor to update the value of our 'foo' property, and to re-render
                 * the block.
                 */
                el(
                    InspectorControls,
                    {},
                    el(TextControl, {
                        label: "URL",
                        value: props.attributes.url,
                        onChange: value => {
                            props.setAttributes({
                                url: value
                            });
                        }
                    }),
                    el(RangeControl, {
                        label: "Max Nights",
                        value: props.attributes["max-nights"],
                        min: "1",
                        max: "30",
                        onChange: value => {
                            props.setAttributes({
                                ["max-nights"]: value
                            });
                        }
                    }),
                    el(RangeControl, {
                        label: "Max Rooms",
                        value: props.attributes["max-rooms"],
                        min: "1",
                        max: "10",
                        onChange: value => {
                            props.setAttributes({
                                ["max-rooms"]: value
                            });
                        }
                    }),
                    el(RangeControl, {
                        label: "Max Adults",
                        value: props.attributes["max-adults"],
                        min: "1",
                        max: "50",
                        onChange: value => {
                            props.setAttributes({
                                ["max-adults"]: value
                            });
                        }
                    }),
                    el(RangeControl, {
                        label: "Max Children",
                        value: props.attributes["max-children"],
                        min: "0",
                        max: "50",
                        onChange: value => {
                            props.setAttributes({
                                ["max-children"]: value
                            });
                        }
                    }),
                    el(RangeControl, {
                        label: "Max Infants",
                        value: props.attributes["max-infants"],
                        min: "0",
                        max: "50",
                        onChange: value => {
                            props.setAttributes({
                                ["max-infants"]: value
                            });
                        }
                    }),
                    el(SelectControl, {
                        label: "Orientation",
                        options: [
                            { label: "Horizontal", value: "horizontal" },
                            { label: "Vertical", value: "vertical" },
                            { label: "Fluid", value: "fluid" }
                        ],
                        value: props.attributes.orientation,
                        onChange: value => {
                            props.setAttributes({
                                orientation: value
                            });
                        }
                    }),
                    el(SelectControl, {
                        label: "Choose Nights or Checkout Date",
                        options: [
                            { label: "Nights", value: "0" },
                            { label: "Checkout Date", value: "1" }
                        ],
                        value: props.attributes["checkout-date"],
                        onChange: value => {
                            props.setAttributes({
                                ["checkout-date"]: value
                            });
                        }
                    }),
                    el(TextControl, {
                        label: "Custom CSS Class",
                        value: props.attributes["css-class"],
                        onChange: value => {
                            props.setAttributes({
                                ["css-class"]: value
                            });
                        }
                    })
                )
            ];
        },

        /**
         * The save function defines the way in which the different attributes should be combined
         * into the final markup, which is then serialized by Gutenberg into `post_content`.
         * @see https://wordpress.org/gutenberg/handbook/block-edit-save/#save
         *
         * @return {Element}       Element to render.
         */
        save: function() {
            return null;
        }
    });
})(window.wp);
